#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define ENTRIES	100 	/* Number of array entries */
#define BUF_SIZE 200 	/* Size of the temporary buffer used for strings */

/* Basic data type used to store student data */
struct student {
	char *firstname;
	char *lastname;
	int aem;
};

typedef struct student studentT;

/* Read the data of `howmany` students and store them
 * in the `data` array. Returns the number of students
 * for which data were actually read (may be less than 
 * howmany in case of errors) */
int readstudents(studentT data[], int howmany) {
	char buffer[BUF_SIZE]; /* Temporary buffers for strings */
	char formatstr[20];	
	int count;
	
	sprintf(formatstr, "%%%ds", BUF_SIZE-1);
	for (count = 0; count < howmany; count++) {
		printf("Student %d\n", count);
		
		printf("Enter firstname: ");
		/* Read to buffer first, then strdup and 
		 * point to the dynamically allocated memory
		 * using the firstname field of the struct */
		scanf(formatstr, buffer);
		data[count].firstname = strdup(buffer);
		/* Check if strdup failed */
		if (data[count].firstname == NULL) {
			printf("Mem alloc problem\n");
			return(count);
		}
		
		printf("Enter lastname: ");
		scanf(formatstr, buffer);
		data[count].lastname = strdup(buffer);
		if (data[count].lastname == NULL) {
			/* Don't forget to free the memory used to
			 * store firstname for the entry where the 
			 * memory allocation (strdup) failure occured */
			free(data[count].firstname);
			printf("Mem alloc problem\n");
			return(count);
		}
		
		printf("Enter AEM: ");
		scanf("%d", &(data[count].aem));
	}
	return(count);
}

/* Free all dynamically allocated memory */
void clean(studentT data[], int howmany) {
	int i;
	
	for (i = 0; i < howmany; i++) {
		free(data[i].firstname);
		free(data[i].lastname);		
	}
}

void print_students(studentT data[], int howmany) {
	int i;
	
	for (i = 0; i < howmany; i++) {
		printf("Student %d: First: %s, Last: %s, AEM: %d\n", 
				i, data[i].firstname, data[i].lastname, data[i].aem);
	}
}

int main(int argc, char *argv[]) {
	studentT database[ENTRIES];
	int howmany, howmany_read;
	
	printf("Howmany students? ");
	scanf("%d", &howmany);
	
	howmany_read = readstudents(database, howmany);
	/* If something went wrong in readstudents, free all 
	 * dynamically allocated memory and exit */
	if (howmany != howmany_read) {
		printf("Exiting\n");
		clean(database, howmany_read);
		exit(1);
	}
	
	print_students(database, howmany);

	/* Free all dynamically allocated memory before exiting */
	clean(database, howmany);
	return(0);
}
